/****************************************************************************
**
** Copyright (C) 2005-2006 Trolltech AS. All rights reserved.
**
** This file is part of the example classes of the Qt Toolkit.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  Please review the following information to ensure GNU
** General Public Licensing requirements will be met:
** http://www.trolltech.com/products/qt/opensource.html
**
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://www.trolltech.com/products/qt/licensing.html or contact the
** sales department at sales@trolltech.com.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#include <QtGui>
#include <QMenu>
#include <QMessageBox>
#include "browser.h"
#include "mainwindow.h"
#include "finder.h"
#include <QRegExp>
#include <QRegExp>
#include <QHeaderView>
#include <elettracolors.h>
#include <macros.h>
#include <QStatusBar>
#include <QMenuBar>
#include <QToolTip>
#include <QSettings>
#include <backupappfinder.h>
#include <browsertreedbusextension.h>

using namespace std;
#include <iostream>

#include <QLabel>
//#include <QtDebug>


BrowserTree::BrowserTree(QWidget *parent)
    : QTreeWidget(parent)
{	
    QStringList labels;
    QString errmess;
    edit_mode = adopt = false;
    x11_unique_check_enabled = true;
    defaultFontSize = 12;
    int i;
    
    QCoreApplication::setOrganizationName("Elettra");
    QCoreApplication::setApplicationName("Browser");

    QSettings settings;
    m_cumbia_apps_names = settings.value("cumbia_exenames").toStringList();

    QString arg;
    filename = qApp->arguments().at(1);
    for(i = 2; i < qApp->arguments().size(); i++)
    {
        arg = qApp->arguments().at(i);
        if(arg == "--edit")
            edit_mode = true;
        else if(arg == "--adopt")
        {
            // qDebug() << "Looking for panels already running...";
            adopt = true;
        }
        else if(arg == "--disable-x11-unique-check")
        {
            x11_unique_check_enabled = false;
            qDebug() << "Check for unique applications disabled.";
        }
        else
        {

        }
    }

    QPalette palette(Qt::blue, Qt::white);
    QToolTip::setPalette(palette);
    
    QFile xmlfile(filename);
    if(edit_mode)
        labels << tr("Browser Element") << tr("Description (editable)") << tr("Executes...") << tr("State") << tr("DBus Service");
    else
        labels << tr("Browser Element") << tr("Description") << tr("Executes...") << tr("State") << tr("DBus Service");


    setIconSize(QSize(22, 22) );
 #if QT_VERSION < 0x050000
    header()->setResizeMode( QHeaderView::ResizeToContents);
 #else
    header()->setSectionResizeMode( QHeaderView::ResizeToContents);
 #endif
    setHeaderLabels(labels);

    header()->hideSection(1);
    header()->hideSection(2);
    header()->hideSection(3);
    header()->hideSection(4);

    bookmarkIcon.addPixmap(style()->standardPixmap(QStyle::SP_FileIcon));
    if (read(&xmlfile))
        setStatusTip(tr("File loaded"));
    else
        exit(EXIT_FAILURE);
    setMouseTracking(true); 
    setContextMenuPolicy(Qt::CustomContextMenu);
    iconmeanings = CreateIconMeaningsMenu();

}

BrowserTree::~BrowserTree()
{
    QSettings s;
    QStringList cumbia_apps;
    foreach(EApplicationLauncher *l, m_appItemHash.keys())
    {
        if(l->property("is_cumbia").toBool() && !cumbia_apps.contains(l->ExeName()))
            cumbia_apps << l->ExeName();
        if(l->AppUtils()->AppliStatus() == NOT_RUNNING)
            delete l;
    }
    s.setValue("cumbia_exenames", cumbia_apps);
    qDebug() << __FUNCTION__ << "stored in cache these cumbia apps " << cumbia_apps;
}

bool BrowserTree::read(QIODevice *device)
{
    QString errorStr;
    int errorLine;
    int errorColumn;

    if (!domDocument.setContent(device, true, &errorStr, &errorLine,
                                &errorColumn)) {
        QMessageBox::information(window(), tr("DOM Elettra Browser"),
                                 tr("Parse error at line %1, column %2:\n%3")
                                 .arg(errorLine)
                                 .arg(errorColumn)
                                 .arg(errorStr));
        return false;
    }

    QDomElement root = domDocument.documentElement();
    if (root.tagName() != "browser") {
        QMessageBox::information(window(), tr("DOM Elettra Browser"),
                                 tr("The file is not an XML Elettra Browser file!"));
        return false;
    } else if (root.hasAttribute("version")
        && root.attribute("version") != "1.0") {
        QMessageBox::information(window(), tr("DOM Elettra Browser"),
                                 tr("The file is not an XML Elettra Browser version 1.0 "
                                    "file."));
        return false;
    }
    
    /* See if a title for the browser is specified in the xml configuration file */
    if(root.hasAttribute("title"))
	wintitle = root.attribute("title");
    else
        wintitle = "BROWSER";

    /* See if an Icon is specified in the xml configuration file */
    if(root.hasAttribute("icon") )
        systray_icon_file = root.attribute("icon") ;
    else
        systray_icon_file = ":icons/elettra_systray.png";

    if(root.hasAttribute("palette_style"))
    {
        qApp->setPalette(EPalette(root.attribute("palette_style")));
        //	    if(root.attribute("palette_style").compare("booster", Qt::CaseInsensitive) == 0)
        //		    qApp->setPalette(EPalette("booster") );
    }

    fontAutoDecrease = false;
    if(root.hasAttribute("fontAutoDecrease"))
    {
        if(root.attribute("fontAutoDecrease") == "yes"
           || root.attribute("fontAutoDecrease") == "true")
            fontAutoDecrease = true;
    }

    if(root.hasAttribute("defaultFontSize"))
        defaultFontSize = root.attribute("defaultFontSize").toInt();


    clear();

    QDomElement child = root.firstChildElement("folder");
    while (!child.isNull()) {
        parseFolderElement(child);
        child = child.nextSiblingElement("folder");
    }

    connect(this, SIGNAL(customContextMenuRequested(const QPoint &) ), 
            this, SLOT(ContextMenu(const QPoint &) ) );

    if(edit_mode) 
    {
        // qDebug() << "Browser in edit mode: you can edit descriptions!";
        connect(this, SIGNAL(itemChanged(QTreeWidgetItem *, int)),
                this, SLOT(updateDomElement(QTreeWidgetItem *, int)));
        connect(this, SIGNAL(itemClicked(QTreeWidgetItem *, int)),
                this, SLOT(SetItemEditable(QTreeWidgetItem *, int)));
    }

    return true;
}

void BrowserTree::mouseDoubleClickEvent(QMouseEvent* e)
{
    QTreeWidgetItem* item;
    int column;
    if(e->type() == QEvent::MouseButtonDblClick)
    {
        item = currentItem();
        column = currentColumn();
        if(item != NULL && column >= 0)
            ExecuteElement(item, column);
        else
            return;
    }
}

bool BrowserTree::write(QIODevice *device)
{
    const int IndentSize = 4;

    QTextStream out(device);
    domDocument.save(out, IndentSize);
    return true;
}

QString BrowserTree::LookForEnvPath(QTreeWidgetItem* item)
{
    QString envpath = "";
    int i = 0;
    QDomElement elem;
    /* Now look up among parents if a folder has the
	 * attribute `envpath="xyz" ' set 
	 */
    QTreeWidgetItem* parent = item; /* first see if ourselves have envpath */
    while(parent != NULL)
    {
        if(i == 0) /* the item itself can contain a "path=" attribute. */
        {
            elem = domElementForItem[parent];
            /* if the element has the attribute path, we return its
			 * value. We look for path only in the current item.
			 */
            if(elem.hasAttribute("path"))
            {
                if(!elem.attribute("path").endsWith('/'))
                    envpath = elem.attribute("path") + '/';
                else
                    envpath = elem.attribute("path");
                return envpath;
            }
        }
        i++;
        elem = domElementForItem[parent];
        /* look for the envpath only in folders. */
        if( elem.tagName() == "folder" && elem.hasAttribute("envpath") )
        {
            envpath = elem.attribute("envpath");
            if(! envpath.endsWith('/') )
                envpath.append('/');
            return envpath;
        }
        parent = parent->parent();
    }
    // qDebug() << "no envpath found in " << i << " parents.";
    return envpath; /* "" */
}

QString BrowserTree::LookForTreeColor(QTreeWidgetItem* item)
{
    QString treecol = "";
    int i = 0;
    QDomElement elem;
    /* Now look up among parents if a folder has the
	 * attribute `envpath="xyz" ' set 
	 */
    QTreeWidgetItem* parent = item; /* first see if ourselves have envpath */
    while(parent != NULL)
    {
        i++;
        elem = domElementForItem[parent];
        if(elem.tagName() == "folder" && elem.hasAttribute("treeColor") )
        {
            treecol = elem.attribute("treeColor");
            return treecol;
        }
        parent = parent->parent();
    }
    // qDebug() << "no envpath found in " << i << " parents.";
    return treecol; /* "" */
}

int BrowserTree::LookForFontSize(QTreeWidgetItem *item)
{
    int size = defaultFontSize;
    int i = 0;
    QDomElement elem;
    /* Now look up among parents if a folder has the
         * attribute `fontSize="n" ' set
         */
    QTreeWidgetItem* parent = item; /* first see if ourselves have fontSize */
    while(parent != NULL)
    {
        i++;
        elem = domElementForItem[parent];
        if(elem.hasAttribute("fontSize") )
        {
            size = elem.attribute("fontSize").toInt();
            return size;
        }
        parent = parent->parent();
        if(fontAutoDecrease)
            size--; /* every time we go up a level we shrink the font */
    }
    return size;
}

int BrowserTree::NumberOfLinks(QDomElement elem)
{
    return elem.elementsByTagName("link").length() ;
}

/* This one is called when the user right clicks on an
 * item on the tree
 */
void BrowserTree::ContextMenu(const QPoint& p)
{
    Q_UNUSED(p);
    unsigned int i;
    QMenu *popupmenu;
    QIcon launch_icon, maximize_icon;
    /* Obtain the selected item */
    QTreeWidgetItem* item = currentItem();
    int column = currentColumn();
    EApplicationLauncher *app = NULL, *master_app = NULL;
    int state;
    QString appstatus, appaction, appname, master_appname, execstring;
    QDomNode node;

    QAction *action = NULL, *dummyaction = NULL, *collapse = NULL, *collapse_all = NULL;

    if(item == NULL || column < 0)
        return;
    popupmenu = new QMenu("Launcher", this);

    QDomElement elem = domElementForItem[item];
    QDomNodeList descendent_elems = elem.elementsByTagName("link");
    if(column == 1)
    {
        popupmenu->addAction(QIcon(":icons/info.png"), "Description");
        popupmenu->addSeparator();
        popupmenu->addAction(item->text(1) );
        popupmenu->exec(QCursor::pos() );
        return;
    }
    if(elem.hasAttribute("executable") && elem.attribute("executable") == "yes")
    {
        //		dummyaction = popupmenu->addAction(QIcon(":icons/choose.png"), QString("Options:") );
        //		popupmenu->addSeparator();
    }
    else if(elem.tagName() == "folder")
    {
        dummyaction = popupmenu->addAction(QIcon(":icons/info.png"),
                                           QString("Folder \"%1\":\nNo actions available.\nClick on \"+\" to explore!")
                                           .arg(item->text(0) ) );
    }
    else if(elem.tagName() == "element")
    {
        dummyaction = popupmenu->addAction(QIcon(":icons/warning.png"),
                                           QString("Element \"%1\":\nNo actions available!")
                                           .arg(item->text(0) ) );
    }
    //	for(i = 0; i < descendent_elems.length(); i++)
    i = 0;
    /* The user can start the default application by the context menu */
    node = (QDomElement) elem;
    while (i < descendent_elems.length() + 1 )
    {
        if(node.isElement() )
        {
            elem = node.toElement();
            execstring = elem.attribute("exename");

            if(i == 0) /* application itself */
            {
                launch_icon = QIcon(":icons/launch.png");
                maximize_icon = QIcon(":icons/maximize.png");
                QString cmdline = LookForEnvPath(currentItem() ) + execstring;
                QString exename;
                QStringList args;
                getExeAndArgs(cmdline, exename, args);
                master_app = GetAppByExeAndArgs(exename, args);
            }
            else
            {
                launch_icon = QIcon(":icons/launch-link.png");
                maximize_icon = QIcon(":icons/maximize_link.png");
            }

            if(elem.tagName() == "element" || elem.tagName() == "folder" ) /* The default element */
                appname = item->text(0); /* read the name directly from the TreeWidgetItem */
            else
                appname = elem.text();

            if(i == 0)
                master_appname = appname;

            /* If the exec string is empty (i.e. app is not available)
			 * we add the element to the context menu,
			 * but we leave it disabled.
			 */
            if(elem.hasAttribute("executable") && elem.attribute("executable") == "yes" && execstring == "")
            {
                appaction = QString("%1 [unavailable]").arg(appname);
                action = popupmenu->addAction(QIcon(":icons/exec_missing.png"), appaction);
                action->setDisabled(true);
            }
            else if(elem.hasAttribute("executable") && elem.attribute("executable") == "yes"
                    && execstring != "")  /* exec string is not empty */
            {
                //	qDebug() << "Element name: " << elem.tagName() << ", executes: " << execstring << ", the so called " <<
                //		appname;
                QString cmdline = LookForEnvPath(currentItem() ) + execstring;
                QString exename;
                QStringList args;
                getExeAndArgs(cmdline, exename, args);
                app = GetAppByExeAndArgs(exename, args);
                if(app == NULL || master_app == NULL)
                {
                    QMessageBox::critical(this, "Error", QString("No application matching the command %1!").arg(execstring ) );
                    return;
                }
                if(app->AppUtils()->AppliStatus() == NOT_RUNNING) {
                    appaction = QString("Start %1").arg(appname );
                    appstatus = "Not Running";
                    state = NOT_RUNNING;
                    action = popupmenu->addAction(launch_icon, appaction);
                    QAction *backupAction = new QAction("Find backup application", popupmenu);
                    backupAction->setProperty("backup", true);
                    backupAction->setData(app->CompleteCommand());
                    connect(backupAction, SIGNAL(triggered()), this, SLOT(executeRaiseOrStartAction()));
                    popupmenu->addAction(backupAction);
                }
                else if(app->AppUtils()->AppliStatus() == RUNNING) {
                    if(!UniqueEnabled() )
                        appaction = "Raise disabled";
                    else
                        appaction = QString("Raise %1").arg(appname );

                    appstatus = "Running";
                    state = RUNNING;
                    action = popupmenu->addAction(maximize_icon, appaction);
                    if(!UniqueEnabled() )
                        action->setEnabled(false);
                    break;
                }
                if(action != NULL)
                    action->setData(QVariant(app->CompleteCommand() ) );
                /* copy/paste executable and parameters into clipboard */
                popupmenu->addSeparator();
                QAction* copyAction = popupmenu->addAction(QIcon(":icons/copy.png"), "Copy command line");
                copyAction->setToolTip(QString("Copy \"%1\"").arg(app->CompleteCommand()));
                copyAction->setData(QVariant(app->CompleteCommand() ) );
                connect(copyAction, SIGNAL(triggered()), this, SLOT(copyCommand()));
            } /* else [exec string is not empty] */
        }
        if(elem.tagName() == "folder")
            break; /* Folders cannot have links */
        node = descendent_elems.at(i);
        i++;
    }
    connect(action, SIGNAL(triggered()), this, SLOT(executeRaiseOrStartAction())) ;
    /* Other actions available in the context menu */
    /* Collapse parent folder, if the item has a parent */
    if(item->parent() != NULL)
    {
        popupmenu->addSeparator();
        collapse = popupmenu->addAction(QIcon(":icons/folder.png"), "Close parent folder");
        connect(collapse, SIGNAL(triggered() ), this, SLOT(collapse_parent() ) );

        collapse_all = popupmenu->addAction(QIcon(":icons/folder.png"), "Close all folders");
        connect(collapse_all, SIGNAL(triggered() ), this, SLOT(collapse_all() ) );
    }
    /* Show/hide columns */
    /*
	popupmenu->addSeparator();
	showdesc = popupmenu->addAction("Show Description");
	showdesc->setCheckable(true);
	showdesc->setChecked(!header()->isSectionHidden(1) );
	connect(showdesc, SIGNAL(triggered() ), this, SLOT(view_description() ) );
	QAction* showstate = popupmenu->addAction("Show State");
	showstate->setCheckable(true);
	showstate->setChecked(!header()->isSectionHidden(3) );
	connect(showstate, SIGNAL(triggered() ), this, SLOT(view_appli_states() ) );
	QAction* showexe = popupmenu->addAction("Show Execution Path");
	showexe->setCheckable(true);
	showexe->setChecked(!header()->isSectionHidden(2) );
	connect(showexe, SIGNAL(triggered() ), this, SLOT(view_exec_command() ) );
	*/

    /* The last option is stop/kill, in a submenu, available on the executable and running items */
    /* We must get the app once again, reading it ftom the currentItem() */
    if(elem.hasAttribute("executable") && elem.attribute("executable") == "yes"
       && execstring != "")  /* exec string is not empty */
    {
        qDebug() << "Menu kill: applisattus di " << master_app->CompleteCommand() <<": "  << master_app->AppUtils()->AppliStatus();
        if(master_app->AppUtils()->AppliStatus() == RUNNING)
        {
            popupmenu->addSeparator();
            QAction *stopact = popupmenu->addAction(QIcon(":icons/exit.png"),
                                                   QString("Close %1").arg(master_appname) );
            stopact->setStatusTip(QString("Close the program \"%1\"").arg(master_appname ) );
            if(master_app->property("dbusExtensionManaged").toBool())
            {
                stopact->setData(master_app->CompleteCommand());
                connect(stopact, SIGNAL(triggered() ), this, SLOT(slotDBusStop()));
            }
            else
            {
                QMenu *killmenu = popupmenu->addMenu(QIcon(":icons/exit.png"), "Stop process menu");
                QAction *stopact = killmenu->addAction(QIcon(":icons/exit.png"),
                                                       QString("Close %1").arg(master_appname) );
                stopact->setStatusTip(QString("Close the program \"%1\"").arg(master_appname ) );
                connect(stopact, SIGNAL(triggered() ), master_app->AppUtils(), SLOT(StopProcess() ) );
                killmenu->addSeparator();
                QAction *killact = killmenu->addAction(QIcon(":icons/int.png"), QString("INTERRUPT %1").arg(master_appname) );
                connect(killact, SIGNAL(triggered() ), master_app->AppUtils(), SLOT(KillINT() ) );
                killact->setToolTip(QString("Close the program with INTERRUPTION signal.\nUse when the above \"Close\" doesn't work") );
                killact->setStatusTip(QString("Close the program with INTERRUPTION signal.\nUse when the above \"Close\" doesn't work") );
                QAction *kill9act = killmenu->addAction(QIcon(":icons/kill.png"), QString("KILL %1 (keep off!)").
                                                    arg(master_appname) );
                connect(kill9act, SIGNAL(triggered() ), master_app->AppUtils(), SLOT(Kill9() ) );
                kill9act->setToolTip("KILL brutally the application.\nUse it only when the two actions above do not work!");
                kill9act->setStatusTip("KILL brutally the application.\nUse it only when the two actions above do not work!");
            }
        }
    }

    popupmenu->exec(QCursor::pos());
}

void BrowserTree::executeRaiseOrStartAction()
{
    QAction *act = static_cast<QAction *>(sender());
    QString args = act->data().toString();
    QTreeWidgetItem* item = currentItem();
    QString envpath = LookForEnvPath(item );

     qDebug() << "BrowserTree::executeRaiseOrStartAction() args vale: " << args << ", text vale: " << item->text(2);
    /* All menu entries are connected to this slot.
	 * So do not try to start elements which do not contain a command */
    if(args != "")
    {
        if(item != NULL && args == envpath + item->text(2) )
        {
            item->setIcon(3, QIcon(":icons/starting.png") );
            item->setText(3, QString("Starting %1...").arg(item->text(0) ) );
            item->setToolTip(0, "Starting...");
        }
        else
        {
            ; // qDebug() << "Item is null";
        }
        setCursor(QCursor(Qt::WaitCursor));
        QString exename;
        QStringList arguments;
        getExeAndArgs(args, exename, arguments);
        if(act->property("backup").toBool()) {
            BackupAppFinder *bf = findChild<BackupAppFinder *>();
            if(!bf)
                  bf = new BackupAppFinder(0);
            bf->exec(exename, envpath);
            exename = bf->exename();
            if(!exename.isEmpty())
                AddApplicationToList(bf->exe_path() + exename + " " + arguments.join(" "), item);
        }
        /* calls Rise() on launcher. Since QTango 4.2, Rise() uses dbus to raise() the application,
         * if the application is registered with DBus sessioin bus.
         */
        if(!exename.isEmpty())
            StartApplication(exename, arguments);
        setCursor(Qt::ArrowCursor);
    }
}

void BrowserTree::copyCommand()
{
    QAction *action = static_cast<QAction *>(sender());
    QClipboard *clipb = QApplication::clipboard();
    clipb->setText(action->data().toString());
}

void BrowserTree::collapse_parent()
{
    QTreeWidgetItem* item = currentItem()->parent();
    if(item == NULL)
    {
        // qDebug() << "The item " << currentItem()->text(0) << " hasn't got a parent!";
        return;
    }
    item->treeWidget()->collapseItem(item);
    setItemSelected(item, true);
    setCurrentItem(item);
}


void BrowserTree::collapse_all()
{
    QList<QTreeWidgetItem *> items = this->findItems("*", Qt::MatchWildcard|Qt::MatchRecursive);
    foreach(QTreeWidgetItem *it, items)
    {
        setItemSelected(it, false);
        if(it->isExpanded() && it->parent() != NULL)
            it->setExpanded(false);
    }
}


//void BrowserTree::ElementClicked(QTreeWidgetItem* item, int column)
//{
//	qDebug() << "Single click";
// 	//item->setStatusTip(0, QString( "Double click to execute %1.").arg(item->text(0) ) );
//}

void BrowserTree::ExecuteElement(QTreeWidgetItem *item, int column)
{
    QString commandLine;
    QDomElement elem = domElementForItem[item];
    qDebug() << __FUNCTION__ << elem.attribute("exename");
    if(elem.hasAttribute("exename") && elem.attribute("exename") == "" )
    {
        if(elem.tagName() == "folder")
            return;
        // qDebug() << "The element " << item->text(0) << " is not available (missing path)";
        QMessageBox::information(this, "Information", QString("The element %1 is not available at the moment.").arg(item->text(0) ) );
        return;
    }
    if(column != 0)
    {
        setStatusTip(QString("Double click on the icon to start %1").arg(item->text(0) ) );
        return;
    }
    if(elem.hasAttribute("executable") && elem.tagName() == "folder" )
    {
        if(elem.attribute("executable") == "yes" )
        {
            QString exename;
            QStringList args;
            commandLine = LookForEnvPath(item) + elem.attribute("exename");
            getExeAndArgs(commandLine, exename, args);
            EApplicationLauncher *launcher = GetAppByExeAndArgs(exename, args);
            if(launcher != NULL && launcher->XUtils()->X11Status() == EXISTING)
                item->setText(3, QString("Raising %1...").arg(item->text(0) ) );
            else if(launcher != NULL)
                item->setText(3, QString("Starting %1...").arg(item->text(0) ) );
            else
                item->setText(3, QString("Item Lookup error (%1)").arg(commandLine));
            item->setToolTip(0, item->text(3));
            item->setIcon(3, QIcon(":icons/starting.png") );
            QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
            qApp->processEvents();
            qDebug() << elem.tagName() << ": " << item->text(0) << "(folder) : eseguo " << exename << args << launcher;
            if(launcher != NULL)
                StartApplication(exename, args);
            QApplication::restoreOverrideCursor();
        }
        else
        {
            ; // qDebug() << "Element " << elem.tagName() <<  ": " << item->text(0) <<  " is not executable";
        }
    }
    else if(elem.hasAttribute("exename") && ( elem.tagName() == "element" || elem.tagName() == "link" ) )
    {
        QString exename;
        QStringList args;
        EApplicationLauncher *launcher;

        commandLine = LookForEnvPath(item) + elem.attribute("exename");
        item->setIcon(3, QIcon(":icons/starting.png") );

        getExeAndArgs(commandLine, exename, args);
        launcher = GetAppByExeAndArgs(exename, args);

        if(launcher != NULL && launcher->XUtils()->X11Status() == EXISTING)
            item->setText(3, QString("Raising %1...").arg(item->text(0) ) );
        else if(launcher != NULL)
            item->setText(3, QString("Starting %1...").arg(item->text(0) ) );
        else
            item->setText(3, QString("Item Lookup error (%1)").arg(commandLine));
        item->setToolTip(0, item->text(3));

        qDebug() << elem.tagName() << ": " << item->text(0) << "(element) : eseguo " << exename << args << launcher;


        printf("\e[1;33m BrowserTree::ExecuteElement: launcher %p exenam %s args %s\e[0m\n", launcher, qstoc(exename), qstoc(args.join(" ")));
        QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
        if(launcher != NULL)
        {
            launcher->setProperty("dbusExtensionManaged", true);
            printf("\e[1;33m BrowserTree::ExecuteElement: calling StartApplication\e[0m\n");
            StartApplication(exename, args);
        }
        QApplication::restoreOverrideCursor();
    }
    else
    {
        ; // qDebug() << "The element " << elem.tagName() << ": " << item->text(0) << " hasn't got the attribute \"executable=\"yes\"|\"no\"\"";
    }
}		

EApplicationLauncher *BrowserTree::GetAppByExeAndArgs(const QString& exeName, const QStringList &args)
{
    foreach(EApplicationLauncher *launcher, m_appItemHash.keys())
    {
         qDebug() << __FUNCTION__ << "comparing  laucnher " << launcher->ExeName() << " with " << exeName <<
                    "and launcehr arg" << launcher->arguments() << "with " << args;
        if(launcher->ExeName() == exeName && launcher->arguments() == args)
            return launcher;
    }
    return NULL;
}

EApplicationLauncher* BrowserTree::GetAppByTreeWidgetItem(QTreeWidgetItem* item)
{
    foreach(QTreeWidgetItem *it, m_appItemHash.values())
        if(it == item)
            return m_appItemHash.key(it);
    return NULL;
}


EApplicationLauncher* BrowserTree::AddApplicationToList(const QString & commandLine, QTreeWidgetItem *item)
{
    QString command = commandLine;
    Q_ASSERT(!command.trimmed().isEmpty());
    QString exename;
    QStringList args;
    /* extract executable and arguments from the complete command line */
    getExeAndArgs(commandLine, exename, args);

    EApplicationLauncher *appli = NULL;
    foreach(EApplicationLauncher *launcher, m_appItemHash.keys())
    {
        if(launcher->ExeName() == exename && launcher->arguments() == args)
        {
            pinfo("BrowserTree::AddApplicationToList: application \"%s\" alread registered", qstoc(command));
            return NULL;
        }
    }
    appli = new EApplicationLauncher(command, false, true, false);
    QStringList qtango3envDisablePrint = QStringList() << "QTANGO_NOPRINT=yes";
    appli->addEnvironmentVariables(qtango3envDisablePrint);
    appli->enableCheckForUnique(x11_unique_check_enabled);
    connect(appli, SIGNAL(ApplicationFinished() ), this, SLOT(ApplicationExited() ) );
    connect(appli, SIGNAL(ApplicationBrutallyFinished()), this, SLOT(ApplicationBrutallyKilled()) );
    connect(appli, SIGNAL(ApplicationRaised(bool) ), this, SLOT(ApplicationRaised(bool) ) );
    m_appItemHash.insert(appli, item);

    return appli;
}

QTreeWidgetItem *BrowserTree::getItemByLauncher(EApplicationLauncher *l)
{
    foreach(EApplicationLauncher * al, m_appItemHash.keys())
    {
        if(al->ExeName() == l->ExeName() && al->arguments() == l->arguments())
            return m_appItemHash.value(al);
    }
    return NULL;
}

void BrowserTree::expand( EApplicationLauncher *eal)
{
    QTreeWidgetItem *it = getItemByLauncher(eal);
    while(it)
    {
        it->setExpanded(true);
        it = it->parent();
    }
}

void BrowserTree::ApplicationBrutallyKilled()
{
    // qDebug() << "Application brutally killed...";
    ApplicationExited(true);
}

void BrowserTree::ApplicationExited(bool brutally, EApplicationLauncher *appLauncher)
{
    QString cu_app;
    int error;
    if(!appLauncher) /* invoked by BrowserTreeDBusExtension */
        appLauncher = (EApplicationLauncher *)sender();

    bool is_cumbia = appLauncher->property("is_cumbia").toBool();
    is_cumbia ? cu_app = "cumbia " : "";

    if(appLauncher->property("dbusExtensionManaged").toBool())
        error = 0;
    else
        error = appLauncher->AppUtils()->ExitCode();

    QIcon folderExecIcon;

    QTreeWidgetItem *item;
    QDomElement elem;

    foreach(EApplicationLauncher *launcher, m_appItemHash.keys())
    {
        if(launcher->ExeName() == appLauncher->ExeName() && launcher->arguments() == appLauncher->arguments())
        {
            item = m_appItemHash.value(launcher);
            break;
        }
    }
    elem = domElementForItem[item];
    if(elem.tagName() == "folder" && error != 0)
    {
        folderExecIcon.addPixmap(QPixmap(":icons/folder_exec_error_open.png"), QIcon::Normal, QIcon::On );
        folderExecIcon.addPixmap(QPixmap(":icons/folder_exec_error.png"), QIcon::Normal, QIcon::Off );
        item->setIcon(0, folderExecIcon );
    }
    else if(elem.tagName() == "element" && error != 0 && NumberOfLinks(elem) > 0)
        item->setIcon(0, QIcon(":icons/exec_more_error.png") );

    else if (elem.tagName() == "element" && error != 0 && NumberOfLinks(elem) == 0)
        item->setIcon(0, QIcon(":icons/exec_error.png") );

    else if(elem.tagName() == "folder" && error == 0)
    {
        folderExecIcon.addPixmap(QPixmap(":icons/folder_exec_open.png"), QIcon::Normal, QIcon::On );
        folderExecIcon.addPixmap(QPixmap(":icons/folder_exec.png"), QIcon::Normal, QIcon::Off );
        item->setIcon(0, folderExecIcon );
    }
    else if(elem.tagName() == "element" && error == 0 && NumberOfLinks(elem) > 0)
        item->setIcon(0, QIcon(":icons/exec_more.png") );

    else if(elem.tagName() == "element" && error == 0 && NumberOfLinks(elem) == 0) {
        if(is_cumbia)
            item->setIcon(0, QIcon(":icons/exec_cumbia.png") );
        else
            item->setIcon(0, QIcon(":icons/exec.png") );
    }


    /* Do not touch first icon if there was no error */
    if(brutally) /* State field */
    {
        item->setIcon(3, QIcon(":icons/warning.png") );
        item->setTextColor(3, Qt::red);
        item->setText(3, QString("The application was brutally killed!" )) ;
        item->setToolTip(0, item->text(3));
        QString mess = QString("The application %1 was brutally killed!").arg(item->text(0));
        //QSystemTrayIcon::Information icon = QSystemTrayIcon::Information;
        ( (MainWindow *)parentWidget() )->TrayIcon()->
                showMessage("Application brutally killed!",
                            mess, QSystemTrayIcon::Warning,  10000);
        QMessageBox::information(this, "Browser", mess);
    }
    else if(error) /* State field */
    {
        item->setIcon(3, QIcon(":icons/warning.png") );
        item->setTextColor(3, Qt::red);
        item->setText(3, QString("The application did not exit cleanly! [code %1]").arg(error) );
        item->setToolTip(0, item->text(3));
        QString mess =  QString("The application %1 did not exit cleanly [code %2]!\n"
                                "Right click on an item and choose \n"
                                "\"Find backup application\" \n"
                                "to try to start an older version.").arg(item->text(0) ).arg(error);
        if(! ( (MainWindow *)parentWidget() )->isVisible() )
            ( (MainWindow *)parentWidget() )->TrayIcon()->
                    showMessage("Unclean exit!", mess,
                                QSystemTrayIcon::Information,  60000);
        else
            QMessageBox::information(this, "Browser", mess);
    }
    else if(!brutally)
    {
        item->setIcon(3, QIcon(":icons/not_running.png") );
        item->setTextColor(3, Qt::gray);
        item->setText(3, cu_app + "not running (successfully stopped).");
        item->setToolTip(0, item->text(3));
    }

}

void BrowserTree::ApplicationRaised(bool ok, EApplicationLauncher *launcher)
{
    QIcon folderExecIcon;
    QTreeWidgetItem *item = NULL;
    QDomElement elem;
    
    if(!launcher) /* launcher not provided (called bu BrowserTreeDBusExtension ) */
        launcher = static_cast<EApplicationLauncher *>( sender());
    bool is_cumbia = launcher->property("is_cumbia").toBool();

    foreach(EApplicationLauncher *lau, m_appItemHash.keys())
    {
        if(lau->ExeName() == launcher->ExeName() && lau->arguments() == launcher->arguments())
        {
            item = m_appItemHash.value(lau);
            break;
        }
    }
    elem = domElementForItem[item];
    if(elem.tagName() == "folder" && !ok)
    {
        folderExecIcon.addPixmap(QPixmap(":icons/folder_exec_error_open.png"), QIcon::Normal, QIcon::On );
        folderExecIcon.addPixmap(QPixmap(":icons/folder_exec_error.png"), QIcon::Normal, QIcon::Off );
        item->setIcon(0, folderExecIcon);
    }

    else if(elem.tagName() == "element" && !ok)
        item->setIcon(0, QIcon(":icons/exec_error.png") );

    else if(elem.tagName() == "folder" && elem.hasAttribute("executable") &&
            elem.attribute("executable") == "yes" && ok)
    {
        folderExecIcon.addPixmap(QPixmap(":icons/folder_exec_open_running.png"), QIcon::Normal, QIcon::On );
        folderExecIcon.addPixmap(QPixmap(":icons/folder_exec_running.png"), QIcon::Normal, QIcon::Off );
        item->setIcon(0, folderExecIcon );
    }
    else if(elem.tagName() == "element" &&  ok && NumberOfLinks(elem) == 0)
        is_cumbia ? item->setIcon(0, QIcon(":icons/exec_running_cumbia.png")) : item->setIcon(0, QIcon(":icons/exec_running.png"));

    else if(elem.tagName() == "element" &&  ok && NumberOfLinks(elem) > 0)
        item->setIcon(0, QIcon(":icons/exec_more_running.png") );

    if(!ok)
    {
        item->setIcon(3, QIcon(":icons/warning.png") );
        item->setTextColor(3, Qt::red);
        item->setText(3, QString("The application did not start!") );
        item->setToolTip(0, "Did not start :(");
        QMessageBox::information(this, "Start an older version?", "Right click on an item and choose \n"
                                                                  "\"Find backup application\" \n"
                                                                  "to try to start an older version.");
    }
    else
    {
        item->setIcon(3, QIcon(":icons/running.png") );
        item->setTextColor(3, Qt::darkGreen);
        item->setText(3, QString("The application is running.") );
        item->setToolTip(0, "Running");
    }
    qDebug() << "Browser tree discovered that the application " << launcher->ExeName() <<  " started." << ok;
    QTimer *check_cumbiaTimer = new QTimer(this);
    check_cumbiaTimer->setSingleShot(true);
    printf("\e[1;36mBrowserTree::ApplicationRaised starting check_cumbiaTimer....\e[0m\n");
    connect(check_cumbiaTimer, SIGNAL(timeout()), this, SLOT(checkIfCumbiaApp()));
    check_cumbiaTimer->setProperty("exename", launcher->ExeName());
    check_cumbiaTimer->setProperty("arguments", launcher->arguments());
    check_cumbiaTimer->start(1500);
}

void BrowserTree::setDBusService(EApplicationLauncher *l, const QString& dbusServiceName)
{
    QTreeWidgetItem *it = getItemByLauncher(l);
    it->setText(4, dbusServiceName);
    QString abbreviated = dbusServiceName;
    it->setToolTip(4, dbusServiceName);
}

void BrowserTree::StartApplication(const QString& executableName, const QStringList & args)
{
    /* Get the pointer to the Application that
	 * corresponds to the execution command specified
	 * by `args'.
	 */
    EApplicationLauncher* app = GetAppByExeAndArgs(executableName, args);

    printf("\e[1;33m BrowserTree::StartApplication: %s %s EApplicationLauncher %p\e[0m\n", qstoc(executableName), qstoc(args.join(" ")),  app);
    qDebug() << __FUNCTION__ << executableName << args << app;
    if(app)
        app->setProperty("dbusExtensionManaged", true);
    // MainWindow::MainWindow creates BrowserTreeDBusExtension with BrowserTree as parent, so this->findChild
    BrowserTreeDBusExtension *treeDBusExtension = findChild<BrowserTreeDBusExtension *>();
    if(treeDBusExtension->dbusRaiseCumbia(executableName + " " + args.join(" "))) {
        printf("\e[0;33m ----> BrowserTree::StartApplication: called dbusRaiseCumbia\n");
    }
    else if(app != NULL) {
        printf("\e[1;33m BrowserTree::StartApplication: calling rise on app launcher %p\e[0m\n", app);
        app->Rise();
    }
    else
        QMessageBox::critical(this, "Error starting application!",
                              QString("Failed to start %1! This is strange!").arg(args.join(" ")) );
    return;
}


void BrowserTree::parseFolderElement(const QDomElement &element,
                                     QTreeWidgetItem *parentItem)
{
    QTreeWidgetItem *item = createItem(element, parentItem);
    QString command;
    EApplicationLauncher * launcher;
    QString title = element.firstChildElement("title").text();
    QString folder_desc = element.firstChildElement("description").text();
    

    if (title.isEmpty())
        title = QObject::tr("Folder"); 
    // for(int h = 0; h < item->columnCount(); h++)
    QFont f = font();
    f.setPointSize(LookForFontSize(item));
    item->setFont(0, f);
    item->setFlags(item->flags() );
    item->setText(0, title);
    //    qDebug() << "font di: " << item->text(0) << ":" << LookForFontSize(item);
    if(folder_desc != "")
        item->setToolTip(0, folder_desc);
    
    item->setStatusTip(0, QString(title) );
    /* Set the description colour similar to the color of the folder */
    item->setTextColor(1, QColor(116, 101, 54) );
    item->setText(1, folder_desc);
    if(edit_mode)
    {	
    	item->setFlags(item->flags() | Qt::ItemIsEditable);
	item->setToolTip(1, "Edit mode: single click to edit, enter to confirm.");
    }
    
    bool folded = (element.attribute("folded") != "no");
    setItemExpanded(item, !folded);
    
    /* Add the application to the list of applications if the folder is executable */
    if( element.hasAttribute("executable") && element.attribute("executable") == "yes" 
        && element.hasAttribute("exename") && element.attribute("exename") != "" )
    {
        /* An executable icon has a folder icon with a gear */
        command = LookForEnvPath(item) + element.attribute("exename");
        /* AddApplicationToList adds the couple Application/item to the pair list */
        launcher = AddApplicationToList(command, item);
        item->setText(2, element.attribute("exename") );
        item->setTextColor(2, Qt::gray);
        if(adopt && launcher->XUtils()->QueryX11Status(command) > 0)
        {
            folderIcon.addPixmap(QPixmap(":icons/folder_exec_open_running.png"), QIcon::Normal, QIcon::On );
            folderIcon.addPixmap(QPixmap(":icons/folder_exec_running.png"), QIcon::Normal, QIcon::Off );
            item->setText(3, "Running (adopted!)");
            item->setIcon(3, QIcon(":icons/running.png") );

        }
        else
        {
            folderIcon.addPixmap(QPixmap(":icons/folder_exec_open.png"), QIcon::Normal, QIcon::On );
            folderIcon.addPixmap(QPixmap(":icons/folder_exec.png"), QIcon::Normal, QIcon::Off );
            item->setText(3, "Not Running");
            item->setToolTip(0, "Not running");
            item->setIcon(3, QIcon(":icons/not_running.png") );
        }
        item->setIcon(0, folderIcon);
        item->setTextColor(3, Qt::gray);
    }
    else if( !element.hasAttribute("executable") || (element.hasAttribute("executable") && 
                                                     element.attribute("executable") == "no") )
    {
        folderExecIcon.addPixmap(QPixmap(":icons/folder-open.png"), QIcon::Normal, QIcon::On );
        folderExecIcon.addPixmap(QPixmap(":icons/folder.png"), QIcon::Normal, QIcon::Off );
        item->setIcon(0, folderExecIcon);
        item->setTextColor(3, Qt::gray);
    }

    QString color = LookForTreeColor(item);
    if(color != "")
        applyColor(item, color);


    QDomElement child = element.firstChildElement();
    while (!child.isNull()) 
    {
        if (child.tagName() == "folder") 
	{
            /* Call ourselves recursively */
            parseFolderElement(child, item);
        }
	else if (child.tagName() == "element") 
	{
            QTreeWidgetItem *childItem = createItem(child, item);
            QString title = child.firstChildElement("title").text();
            QString description = child.firstChildElement("description").text();
            if (title.isEmpty())
                title = QObject::tr("Folder");

            //	    for(int h = 0; h < childItem->columnCount(); h++)

            childItem->setFlags(item->flags() );
	    childItem->setText(0, title);
            //	    qDebug() << "font di: " << childItem->text(0) << ":" << LookForFontSize(childItem);
        QFont f = font();
        f.setPointSize(LookForFontSize(childItem));
        childItem->setFont(0, f);
	    /* Set the element description colour similar to the element color */
            childItem->setTextColor(1, QColor(135, 157, 195) );
            childItem->setText(1,description);

	    if(description != "")
                childItem->setToolTip(0, description);
	    
    	    childItem->setStatusTip(0, QString(title) );
	    /* Add the application to the list */
	    if(child.hasAttribute("exename") )
	    {
                if(child.attribute("exename") != "")
                {
                    command = LookForEnvPath(childItem) + child.attribute("exename");
                    launcher = AddApplicationToList(command, childItem );
                    bool is_cumbia = false;
                    if(launcher) // launcher null for duplicated entries
                        is_cumbia = m_cumbia_apps_names.contains(launcher->ExeName());
                    QString cu_app;
                    is_cumbia ? cu_app = "cumbia app" : "app";
                    // from a QSettings file we cache cumbia applications
                    if(launcher)
                        launcher->setProperty("is_cumbia", is_cumbia);
                    if(adopt && launcher && launcher->XUtils()->QueryX11Status(command) > 0)
                    {
                        childItem->setText(3, "Running (adopted!)");
                        childItem->setIcon(3, QIcon(":icons/running.png") );
                        if(NumberOfLinks(child) > 0 )
                            childItem->setIcon(0, QIcon(":icons/exec_more_running.png") );
                        else
                            is_cumbia ? childItem->setIcon(0, QIcon(":icons/exec_running_cumbia.png") ) : childItem->setIcon(0, QIcon(":icons/exec_running.png") );
                    }
                    else
                    {
                        childItem->setText(3, cu_app + " not Running");
                        childItem->setToolTip(0, cu_app + " not Running");
                        childItem->setIcon(3, QIcon(":icons/not_running.png") );
                        if(NumberOfLinks(child) > 0 )
                            childItem->setIcon(0, QIcon(":icons/exec_more.png") );
                        else if(!is_cumbia)
                            childItem->setIcon(0, QIcon(":icons/exec.png") );
                        else if(is_cumbia)
                            childItem->setIcon(0, QIcon(":icons/exec_cumbia.png") );
                    }
                    childItem->setTextColor(3, Qt::gray);
                    item->setTextColor(2, Qt::gray);
                    childItem->setText(2, child.attribute("exename") );
                    childItem->setTextColor(3, Qt::gray);
                }
                else
                {
                    childItem->setIcon(0, QIcon(":icons/exec_missing.png"));
                    childItem->setFlags(0);
                }
	    }
	    QString color = LookForTreeColor(childItem);
	    if(color != "")
                applyColor(childItem, color);


        }
	/* We suppose that the link elements contain the same application 
	 * path of the executable they point to.
	 * So we do not care parsing them here and registering them in the
	 * application list.
	 */
	else if (child.tagName() == "separator") 
	{
            QTreeWidgetItem *childItem = createItem(child, item);
            childItem->setFlags(item->flags() & ~(Qt::ItemIsSelectable ));
            childItem->setText(0, QString(30, 0xB7));
        }
        child = child.nextSiblingElement();
    }
}

QTreeWidgetItem *BrowserTree::createItem(const QDomElement &element,
                                         QTreeWidgetItem *parentItem)
{
    QTreeWidgetItem *item;
    if (parentItem) 
    {
        item = new QTreeWidgetItem(parentItem);
    }
    else
    {
        item = new QTreeWidgetItem(this);
    }
    domElementForItem.insert(item, element);
    return item;
}

void BrowserTree::view_exec_command(bool en)
{
    en ? header()->showSection(2) : header()->hideSection(2);
}

void BrowserTree::view_appli_states(bool en)
{
    en ? header()->showSection(3) : header()->hideSection(3);
}

void BrowserTree::view_description(bool en)
{
    en ? header()->showSection(1) : header()->hideSection(1);
}

void BrowserTree::viewDBusService(bool en)
{
    en ? header()->showSection(4) : header()->hideSection(4);
}

void BrowserTree::unlockHeaders(bool en)
{
#if QT_VERSION < 0x050000
    en ? header()->setResizeMode(QHeaderView::Interactive) : header()->setResizeMode(QHeaderView::ResizeToContents);
#else
    en ? header()->setSectionResizeMode(QHeaderView::Interactive) : header()->setSectionResizeMode(QHeaderView::ResizeToContents);
#endif
}

QMenu* BrowserTree::CreateIconMeaningsMenu()
{
    QMenu *im, *fm, *em, *mm; /* icon meanings, folder meanings, executable meanings, misc meanings */
    im = new QMenu(this);

    fm = im->addMenu(QIcon(":icons/folder.png"), "Folders");
    fm->addAction(QIcon(":icons/folder_exec.png"), "This is a closed executable folder not running:"
                  "\ndouble click to start it or click on \"+\" to expand it!" );
    fm->addAction(QIcon(":icons/folder_exec_open.png"), "Like the former, but expanded: click on \"+\" to collapse it!");
    fm->addAction(QIcon(":icons/folder_exec_running.png"), "A closed executable folder whose represented executable is running.");
    fm->addAction(QIcon(":icons/folder_exec_open_running.png"), "An open executable folder with its program running" );
    fm->addAction(QIcon(":icons/folder_exec_error.png"), "An executable folder, but there was an error starting/stopping"
                  " it: click on \"View\"->\"View App States\" for details.");
    fm->addAction(QIcon(":icons/folder_exec_error_open.png"), "Like the former, but open.");

    im->addSeparator();
    em = im->addMenu(QIcon(":icons/gear.png"), "Executables");
    em->addAction(QIcon(":icons/exec.png"), "This represents an executable program not running");
    em->addAction(QIcon(":icons/exec_running.png"), "This represents an executable program running");
    em->addAction(QIcon(":icons/exec_more.png"), "If you right-click on this, you can start other executables.");
    em->addAction(QIcon(":icons/exec_more_running.png"), "Like above, but the main application is running.");
    em->addAction(QIcon(":icons/exec_more_error.png"), "The main application represented has had problems starting/stopping.");
    em->addAction(QIcon(":icons/exec_missing.png"), "The executable is not available, although this represents a member node.");

    im->addSeparator();
    mm = im->addMenu(QIcon(":icons/other.png"), "Other emblems");
    mm->addAction(QIcon(":icons/running.png"), "The application is running.");
    mm->addAction(QIcon(":icons/not_running.png"),  "The application is not running/has been stopped. Available in \"State\" column.");
    mm->addAction(QIcon(":icons/warning.png"), "The application did not start or stop correctly. Available in \"State\" column.");
    mm->addAction(QIcon(":icons/starting.png"), "Appears right clicking on an item: starts the item itself.");
    mm->addAction(QIcon(":icons/launch-link.png"), "Appears right clicking on an item: starts a contextual node (link)");
    mm->addAction(QIcon(":icons/maximize.png"), "Appears right clicking on an item: maximizes a running application");
    mm->addAction(QIcon(":icons/maximize_link.png"), "Appears right clicking on an item: maximizes a running application link");
    mm->addAction(QIcon(":icons/starting.png"), "An application is being started or maximized. Available in \"State\" column.");
    return im;
}

void BrowserTree::updateDomElement(QTreeWidgetItem *item, int column)
{
    // qDebug() << "In update dom element" ;
    QDomElement element = domElementForItem.value(item);
    if (!element.isNull()) 
    {
        if (column == 1) 
	{    
            QDomElement oldTitleElement = element.firstChildElement("description");
            QDomElement newTitleElement = domDocument.createElement("description");

            QDomText newDescText = domDocument.createTextNode(item->text(1));
            newTitleElement.appendChild(newDescText);

            element.replaceChild(newTitleElement, oldTitleElement);
        } 
    }
}

QStringList BrowserTree::runningApplications() const
{
    QStringList list;
    foreach(EApplicationLauncher *launcher, m_appItemHash.keys())
    {
        if(launcher->AppUtils()->AppliStatus() == RUNNING)
        {
            QString app = launcher->ExeName();
            foreach(QString arg, launcher->arguments())
                app += " " + arg;
            list << app;
        }
    }
    return list;
}

void BrowserTree::SetItemEditable(QTreeWidgetItem * item, int column)
{
    if (column == 1)
        editItem(item, column);
}

int BrowserTree::ShowFindDialog()
{
    int ret = 0;
    finder = new Finder(this);
    finder->exec();
    QString expr = finder->Expression();

    if(expr != "" )
    {
        if((ret = LookFor(expr) ) > 0)
            finder->AddToHistory(expr, true);
        else
            finder->AddToHistory(expr, false);
    }
    return ret;
}

void BrowserTree::getExeAndArgs(const QString& command, QString& exename, QStringList& arguments)
{
    /* reset refernce arguments */
    exename = QString();
    arguments.clear();
    if(command.contains(QRegExp("\\s+")))
    {
        arguments = command.split(QRegExp("\\s+"), QString::SkipEmptyParts);
        exename = arguments.first();
        arguments.removeFirst();
    }
    else
    {
        exename = command;
    }
    if(exename.contains("/")) /* exename with path: extract exename */
        exename = exename.split("/", QString::SkipEmptyParts).last();
}

int BrowserTree::LookFor(QString expr)
{
    int trovato = 0;
    int i;
    QTreeWidgetItem *item;
    QList <QTreeWidgetItem* > items;
    /*
  	if(finder->CaseSensSearch() )
		qDebug() << "regexp con case sensitive search";
	else
		qDebug() << "regexp senza case sensitive search";
*/
    QRegExp regexp(expr, finder->CaseSensSearch(), QRegExp::Wildcard);
    /* If the user chooses regular expression... */
    if(finder->SearchRegexp() )
        regexp.setPatternSyntax(QRegExp::RegExp2);
    items = domElementForItem.uniqueKeys();
    QDomElement elem;
    for(i = 0; i < items.size(); i++)
    {
        if(items[i]->text(0).contains(regexp ) )
        {

            elem = domElementForItem.value(items[i]);

            /* Search among applications and find an item which is not executable: continue! */
            if(!elem.isNull()  && finder->ExeSearch() &&  (
                    !elem.hasAttribute("executable") || (elem.hasAttribute("executable") &&
                                                         elem.attribute("executable") != "yes" ) ) )
            {
                continue;
            }
            /* Search among running applications and find an application not running...
			 * continue! 
			 */
            else if(!elem.isNull()  && finder->ExeSearch() && elem.hasAttribute("executable") &&
                    elem.attribute("executable") == "yes" && finder->SearchRunning() )
            {
                if(GetAppByTreeWidgetItem(items[i]) == NULL)
                    continue;
                else if(GetAppByTreeWidgetItem(items[i])->AppUtils()->AppliStatus() != RUNNING)
                    continue;
            }
            /* Search among not running applications and find an application running: continue! */
            else if(!elem.isNull()  && finder->ExeSearch() && elem.hasAttribute("executable") &&
                    elem.attribute("executable") == "yes" && finder->SearchNotRunning() )
            {
                if(GetAppByTreeWidgetItem(items[i]) == NULL)
                    continue;
                else if(GetAppByTreeWidgetItem(items[i])->AppUtils()->AppliStatus() == RUNNING)
                    continue;
            }

            trovato++;
            item = items[i];
            if(item)
                item->setSelected(true);
            /* We have to expand all parents before selecting our item */
            while( ( item = item->parent() ) != NULL)
            {
                item->setExpanded(true);
            }
            if(item != NULL)
                item->setSelected(true);
        }
        else
            items[i]->setSelected(false);
    }
    return trovato;
}

void BrowserTree::slotDBusStop()
{
    qprintf("\e[1;34memetto dbusStop %s\e[0m\n", qstoc(static_cast<QAction*>(sender())->data().toString()));
    emit dbusStop(static_cast<QAction*>(sender())->data().toString());
}

void BrowserTree::checkIfCumbiaApp()
{
    QString exe = sender()->property("exename").toString();
    QStringList args = sender()->property("arguments").toStringList();
    bool is_cumbia = findChild<BrowserTreeDBusExtension *>()->isCumbiaApp(exe);
    EApplicationLauncher* l = GetAppByExeAndArgs(exe, args);
    if(l)
    {
        l->setProperty("is_cumbia", is_cumbia);
        if(is_cumbia) {
            QTreeWidgetItem *it = getItemByLauncher(l);
            if(!it->toolTip(0).startsWith("cumbia"))
                it->setToolTip(0, "cumbia " + it->toolTip(0));
            it->setIcon(0, QIcon(":icons/exec_running_cumbia.png"));
        }
    }
    sender()->deleteLater();
}

void BrowserTree::slotDBusRaise()
{
    emit dbusRaise(static_cast<QAction*>(sender())->data().toString());
}

void BrowserTree::applyColor(QTreeWidgetItem *it, QString color)
{
    QColor col;
    if(color == "pColor")
        col = EColor(Elettra::yellow);
    else if(color == "ptbColor")
        col = EColor(Elettra::greenWater);
    else if(color == "bColor")
        col = EColor(Elettra::cyan);
    else if(color == "btsColor")
        col = EColor(Elettra::darkWater);
    else if(color == "srColor")
        col = EColor(Elettra::green);
    else
    {
	    col.setNamedColor(color);
	    if(!col.isValid())
		    perr("Error: color \"%s\" is not valid", qstoc(color));
    }

//    QBrush brush(col, Qt::Dense4Pattern);
    for(int i = 0; i < it->columnCount(); i++)
        it->setBackground(i, col);
}

