#include "configurationfilewatcher.h"
#include <QFile>
#include <QTextStream>
#include <QTimer>

ConfigurationFileWatcher::ConfigurationFileWatcher(QObject *parent, const QString& filename) :
        QObject(parent)
{
    m_fileName = filename;
    QTimer *timer = new QTimer(this);
    timer->setSingleShot(false);
    timer->setInterval(10000);
    connect(timer, SIGNAL(timeout()), this, SLOT(refresh()));
    m_fileContents = m_readFile();
    timer->start();
}

void ConfigurationFileWatcher::refresh()
{
    QString oldContents = m_fileContents;
    m_fileContents = m_readFile();
    if(m_fileContents != oldContents)
        emit fileChanged();
}

QString ConfigurationFileWatcher::m_readFile()
{
    QString ret;

    QFile file(m_fileName);
    if(!file.open(QIODevice::ReadOnly | QIODevice::Text))
    {
        m_errorMsg = file.errorString();
        emit error(m_errorMsg);
    }
    else
    {
        QTextStream in(&file);
        while(!in.atEnd())
            ret += in.readLine();
    }
    return ret;
}
